#!/usr/bin/perl
# process-word-and-file-list.pl word-list file-list
# word-list is of the form:
#
# wordset
# replacement\t\tbadword
# replacement\t\tbadwerd
#
# word\t\twerd
#
# in the first case, wordset is used as the patch name,
# in the second case, word is used.
#
# file-list is a simple list of files, one file per line
#

my ($wordlist, $filelist) = @ARGV;
my ($wordset) = ('');
my @comment = ();
my @files = ();
my %filelists = ();
my %wordwork = ();
my %wordfiles = ();

sub get_target_files {
  open FILES, '<', $filelist || die "nothing to do";
  foreach my $file (<FILES>) {
    chomp($file);
    push @files, $file;
  }
  close FILES;
}

my $workfile = "new.$$";

sub do_work {
  foreach my $wordset (sort keys %wordfiles) {
    my @comment = @{$wordwork{$wordset}};
    my $comment = join "", sort { my $aa = lc $a; my $bb = lc $b; $aa cmp $bb || $a cmp $b; } @comment;
    my $commands = '';
    foreach my $word (@comment) {
      $word =~ /^(.*?)\t\t(.*)$/;
      my ($word, $repl) = ($2, $1);
      $word =~ s/([.\\*+{}()\[\]])/\\$1/g;
      $commands .= "my (\$word, \$repl) = (q!$word!, q!$repl!); s{(\\b|_)\$word(\\b|_)}{\$1\$repl\$2}g; ";
    }
    my @files = keys %{$wordfiles{$wordset}};
    foreach my $file (@files) {
      my @lines = ();
      if (open (INPUT, '<', $file)) { 
        my $dirty = 0;
        my $line;
        while (!$dirty && ($line = <INPUT>)) {
          $clean = $line;
          local $_ = $line;
          if ($line) {
            eval "$commands";
            $dirty = $_ ne $line;
          }
          push @lines, $_;
          if ($dirty) {
            open (OUTPUT, '>', $workfile);
            print OUTPUT @lines;
          }
        }
        if ($dirty) {
          while (local $_ = <INPUT>) {
            eval "$commands";
            print OUTPUT $_;
          }
          close OUTPUT;
          rename $workfile, $file;
        }
        close INPUT;
      }
    }
    if ($wordset =~ /^(?:status|series)$/) {
      $wordset .= '_';
    } else {
      $wordset =~ s/[^a-z0-9]/_/gi;
    }
    system('hg', 'qnew', $wordset, '-m', "replacements for $wordset:
$comment");
  }
}

sub get_work {
  open WORDS, '<', $wordlist || die "nothing to do";
  my $comments = [];
  while (1) {
    my $word = <WORDS>;
    unless ($word =~ /./) {
      if (@$comments) {
        $wordwork{$wordset} = $comments;
      }
      $wordset = '';
      $comments = [];
      last if eof(WORDS);
      next;
    }
    unless ($wordset) {
      $word =~ /^(.*?)\t|^(.*)$/;
      $wordset = $1 || $2;
      next if $2;
      $wordset = lc $wordset;
    }
    next if $wordset =~ /unresolved-unresolved/;
    $word =~ /^(.*?)\t\t(.*)$/;
    my ($replacement, $searchtext) = ($1, $2); 
    next if $replacement eq $searchtext;
    push @$comments, $word;
  }
  close WORDS;
}

sub add_work {
  my ($file, $wordset) = @_;
  my $files = {};
  my $key = $wordfiles{$wordset};
  $files = $key if defined $key;
  $$files{$file} = 1;
  $wordfiles{$wordset} = $files;
  return 1;
}

sub build_scan_string {
  my $commands = '';
  foreach my $wordset (keys %wordwork) {
    my @comment = @{$wordwork{$wordset}};
    my @bad = ();
    foreach my $word (@comment) {
      $word =~ /^(.*?)\t\t(.*)$/;
      $word = $2;
      $word =~ s/([.\\*+{}()\[\]])/\\$1/g;
      push @bad, $word;
    }
    $commands .= "\$_ =~ m!(?:\\b|_)(?:".join("|", @bad).")(?:\\b|_)! && add_work(\$file, q<$wordset>); ";
  }
  return $commands;
}

sub scan_files {
  my $commands = build_scan_string();
  local $/ = undef;
  foreach my $file (@files) {
    open INPUT, '<', $file;
    local $_ = <INPUT>;
    eval "$commands"; 
    close INPUT;
  }
}

get_target_files();
get_work();
scan_files();
do_work();
